#include <mega32.h>
#include <stdio.h>
#include <delay.h>
#include <alcd.h>
#include <string.h>
#include "TWI.h"
#include "DS1307.h"

char SHOW_MODE=0;
extern const char Week_Days[7][20];
extern const char DEC_Format[];
extern const char Hex_Format[];

extern const char STR_TWO_DOT[];
extern const char STR_20[];
extern const char STR_SLASH[];

void RTC_DS1307_Init(void)
{
  TWI_Init();
}

void RTC_DS1307_WR_DATA(unsigned char ADD,unsigned char DATA)
{
  TWI_Start();  
  TWI_Write_Byte(DS1307_WR);  
  TestAck(MT_SLA_ACK); 
  TWI_Write_Byte(ADD); 
  TestAck(MT_DATA_ACK);  
  TWI_Write_Byte(DATA);    
  TestAck(MT_DATA_ACK);  
  TWI_Stop(); 
  delay_ms(5);
}

unsigned char RTC_DS1307_RD_DATA(unsigned char ADD)
{
  unsigned char REC_Data;
  TWI_Start(); 
  TWI_Write_Byte(DS1307_WR);    
  TestAck(MT_SLA_ACK);   
  TWI_Write_Byte(ADD); 
  TestAck(MT_DATA_ACK);     
  TWI_ReStart(); 
  TWI_Write_Byte(DS1307_RD); 
  TestAck(MR_SLA_ACK);
  REC_Data= TWI_Read_Byte();  
  SetAck();     
  delay_ms(5);
  TWI_Stop();  
  delay_ms(5);  
  return REC_Data;
}

void RTC_DS1307_Set_Time(unsigned char Hour,unsigned char Minute,unsigned char Second)
{
  RTC_DS1307_WR_DATA(DS1307_Second_ADD,DEC_To_BCD(Second));
  RTC_DS1307_WR_DATA(DS1307_Minute_ADD,DEC_To_BCD(Minute));
  RTC_DS1307_WR_DATA(DS1307_Hour_ADD,DEC_To_BCD(Hour));
}

void RTC_DS1307_Set_Date(unsigned char Year,unsigned char Month,unsigned char Date)
{
  RTC_DS1307_WR_DATA(DS1307_Date_ADD,DEC_To_BCD(Date));
  RTC_DS1307_WR_DATA(DS1307_Month_ADD,DEC_To_BCD(Month));
  RTC_DS1307_WR_DATA(DS1307_Year_ADD,DEC_To_BCD(Year));
}

void RTC_DS1307_Set_DOW(unsigned char Day)
{
  RTC_DS1307_WR_DATA(DS1307_Day_ADD,Day);
}

void RTC_DS1307_Get_Time(unsigned char *Hour,unsigned char *Minute,unsigned char *Second)
{
  *Second = RTC_DS1307_RD_DATA(DS1307_Second_ADD)&0x7F;
  *Minute = RTC_DS1307_RD_DATA(DS1307_Minute_ADD);
  *Hour   = RTC_DS1307_RD_DATA(DS1307_Hour_ADD)&0x3F;
}

void RTC_DS1307_Get_Date(unsigned char *Year,unsigned char *Month,unsigned char *Date,unsigned char *Day)
{
  *Day   = RTC_DS1307_RD_DATA(DS1307_Day_ADD)&0x07; 
  *Date  = RTC_DS1307_RD_DATA(DS1307_Date_ADD)&0x3F;
  *Month = RTC_DS1307_RD_DATA(DS1307_Month_ADD)&0x1F;
  *Year  = RTC_DS1307_RD_DATA(DS1307_Year_ADD);
}

void RTC_DS1307_Display_Time(void)
{
  unsigned char H,M,S;
  char str_S[3],str_M[3],str_H[3];  
  char Time[12];    
  Time[0]='\0';
  
  RTC_DS1307_Get_Time(&H,&M,&S);  
  
  sprintf(str_S,Hex_Format,S);
  sprintf(str_M,Hex_Format,M);
  sprintf(str_H,Hex_Format,H);  
  
  strcat(Time,str_H); 
  strcatf(Time,STR_TWO_DOT);
  strcat(Time,str_M); 
  strcatf(Time,STR_TWO_DOT);
  strcat(Time,str_S);   
  
  LCD_WR_STR(0,0,Time);  
  Time[0]='\0';
}

void RTC_DS1307_Display_Date(void)
{
  unsigned char Y,M,D,DOW;
  char str_Y[3],str_M[3],str_D[3];  
  char Date[12];    
  Date[0]='\0';
  
  RTC_DS1307_Get_Date(&Y,&M,&D,&DOW);  
  
  sprintf(str_Y,Hex_Format,Y);
  sprintf(str_M,Hex_Format,M);
  sprintf(str_D,Hex_Format,D);  
  
  strcatf(Date,STR_20);
  strcat(Date,str_Y); 
  strcatf(Date,STR_SLASH);
  strcat(Date,str_M); 
  strcatf(Date,STR_SLASH);
  strcat(Date,str_D);   
  
  LCD_WR_STR(0,0,Date);  
  Date[0]='\0';    
}

void RTC_DS1307_Display_Day_Week(void)
{
  unsigned char Y,M,D,DOW;
  RTC_DS1307_Get_Date(&Y,&M,&D,&DOW);
  switch(DOW)
  {
    case 1:
      LCD_WR_STRF(0,0,Week_Days[0]);  
      break;
    case 2:
      LCD_WR_STRF(0,0,Week_Days[1]);  
      break;
    case 3:
      LCD_WR_STRF(0,0,Week_Days[2]);  
      break;
    case 4:
      LCD_WR_STRF(0,0,Week_Days[3]);  
      break;
    case 5:
      LCD_WR_STRF(0,0,Week_Days[4]);  
      break; 
    case 6:
      LCD_WR_STRF(0,0,Week_Days[5]);  
      break;
    case 7:
      LCD_WR_STRF(0,0,Week_Days[6]);  
      break;           
  }
}

void RTC_DS1307_Display_Date_Time(void)
{
  if(SHOW_MODE<5)
	{
	  RTC_DS1307_Display_Time();
	}
	if(SHOW_MODE>=5 && SHOW_MODE<=6)
	{
	  RTC_DS1307_Display_Date();	
	}
	if(SHOW_MODE>=7 && SHOW_MODE<=9)
	{
	  RTC_DS1307_Display_Day_Week();
	}
	SHOW_MODE++;
	if(SHOW_MODE==9)
	{
	  SHOW_MODE=0;
	}
}

unsigned char DEC_To_BCD(unsigned char DEC)
{
  return ( (DEC/10*16) + (DEC%10) );
}

unsigned char BCD_To_DEC(unsigned char BCD)
{
  return ( (BCD/16*10) + (BCD%16) );
}